<?php

namespace Ebuy\Ebuy\Providers;

use Illuminate\Support\ServiceProvider;

class EbuyServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'ebuy');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'ebuy');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerEbuy();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerEbuy()
    {
        $this->app->bind('ebuy', function($app) {
            return new Ebuy($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Ebuy', 'Lavalite\Ebuy\Facades\Ebuy');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('ebuy.ebuy', function ($app) {
            return $this->app->make('Ebuy\Ebuy\Ebuy');
        });

        // Bind Person to repository
        $this->app->bind(
            'Ebuy\Ebuy\Interfaces\PersonRepositoryInterface',
            \Ebuy\Ebuy\Repositories\Eloquent\PersonRepository::class
        );
        // Bind Person to repository
        $this->app->bind(
            'Ebuy\Ebuy\Interfaces\PersonRepositoryInterface',
            \Ebuy\Ebuy\Repositories\Eloquent\PersonRepository::class
        );

        $this->app->register(\Ebuy\Ebuy\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Ebuy\Ebuy\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and ebuy's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'ebuy.ebuy'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeEbuy::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['ebuy.ebuy'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('ebuy/ebuy.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/ebuy')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/ebuy')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
